'use strict';

angular.module('ng-laravel').service('ProductService', function($rootScope, Restangular,CacheFactory) {
    
	/*
     * Build collection /product
     */
	var _productService = Restangular.all('product');
    
	if (!CacheFactory.get('productsCache')) {
		var productsCache = CacheFactory('productsCache');
    }

    /*
     * Get list of products from cache.
     * if cache is empty, data fetched and cache create else retrieve from cache
     */
    this.cachedList = function() {
        // GET /api/product
		if (!productsCache.get('list')) {
			return this.list();
        } else{
			return productsCache.get('list');
        }
    };

    /*
     * Get list of products
     */
    this.list = function() {
		// GET /api/product
        var data = _productService.getList();
        productsCache.put('list', data);
        return data;
    };

	/*
     * Pagination change
     */
    this.pageChange = function(pageNumber, per_page, end, pagination) {	
        // GET /api/product?page=2		
		if(end == pagination) {
			//pageNumber = 1;
			return _productService.getList({page:pageNumber, per_page:per_page});
		} else {
			return _productService.getList({page:pageNumber,per_page:per_page});
		}
    };

    /*this.cachedShow = function(id) {
        // GET /api/task/:id
        if (!productsCache.get('show'+id)) {
            return this.show(id);
        } else{
            return productsCache.get('show'+id);
        }
    };*/

	/*
     * Search in product
     */
	this.search = function(query, per_page, pageNumber, end, pagination) {
	    // GET /api/product/search?query=test&per_page=10
		//alert(JSON.stringify(query));
		//alert(JSON.stringify(pageNumber));
        if(query !=''){
			if(end == pagination) {
				pageNumber = 1;
				return _productService.customGETLIST("search", {page:pageNumber, query:query, per_page:per_page});
			} else {
				return _productService.customGETLIST("search", {page:pageNumber, query:query, per_page:per_page});	
			}
        } else {
			return _productService.getList({page:pageNumber,per_page:per_page});
        }
    }
	
	/*
     * pushall Product (PUT)
     */
    this.pushall = function() {        
		Restangular.all('product/pushall').post().then(function(response) {
			$rootScope.$broadcast('product.pushall', response.skuconsumed);
        }, function(response) {			
			$rootScope.$broadcast('product.validationError', response.data.error);
        });
    };

	/*
     * push Product 
     */
    this.push = function(selection) {
		Restangular.several('product/push',selection).post().then(function(response) {
			$rootScope.$broadcast('product.push', selection, response.skuconsumed);
        }, function(response){
			//console.log(response);
			$rootScope.$broadcast('product.validationError', response.data.error);
        });
    };	

	/*
     * unblock Product 
     */
    this.unblock = function(selection) {      
		Restangular.several('product/unblock',selection).post().then(function() {
			$rootScope.$broadcast('product.unblock', selection);
        }, function(response){			
			$rootScope.$broadcast('product.validationError', response.data.error);
        });
    };

	/*
     * block Product
     */
    this.block = function(selection) {  
	    Restangular.several('product/block', selection).post().then(function() {
		    $rootScope.$broadcast('product.block', selection);
        }, function(response){
			$rootScope.$broadcast('product.validationError',response.data.error);
        });
    };

	this.fetchStatus = function() {
        // GET /api/ebconfig/status
		var _ebconfigService = Restangular.one('ebconfig');
        var data = _ebconfigService.customGET("status");  
		return data;
    };

	this.updatePDesc = function(data) {		
	    Restangular.all('product').customPOST(data, "updatepdesc").then(function() {
		    $rootScope.$broadcast('product.updatepdesc', data);
        }, function(response){
			$rootScope.$broadcast('product.validationError', response.data.error);
        });
    };

	/*
     * reimport Product 
     */
    this.reimport = function(selection) {
		Restangular.several('product/reimport',selection).post().then(function(response) {
			$rootScope.$broadcast('product.reimport', selection);
        }, function(response){
			$rootScope.$broadcast('product.validationError', response.data.error);
        });
    };

	this.remove = function(selection) {
		Restangular.several('product/remove',selection).post().then(function(response) {
			$rootScope.$broadcast('product.remove', selection);
        }, function(response){
			$rootScope.$broadcast('product.validationError', response.data.error);
        });
    };

	this.getVariants = function(product_id) {
		// GET /api/product/variant
		var res = _productService.customGETLIST("variants", {product_id:product_id});
		return res;
    };
});